package cz.muni.fi.pa165.dao;

import cz.muni.fi.pa165.PersistenceSampleApplicationContext;
import cz.muni.fi.pa165.dao.match.MatchDao;
import cz.muni.fi.pa165.entity.MatchEntity;
import cz.muni.fi.pa165.entity.TeamEntity;
import cz.muni.fi.pa165.entity.TournamentEntity;
import cz.muni.fi.pa165.enums.CountryEnum;
import cz.muni.fi.pa165.enums.TournamentFormat;
import org.assertj.core.api.Assertions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.testng.AbstractTransactionalTestNGSpringContextTests;
import org.testng.annotations.Test;

import java.util.HashSet;
import java.util.List;
import java.util.UUID;

import static org.assertj.core.api.AssertionsForClassTypes.assertThat;
/**
 * @author Adrian Piacek
 */
@ContextConfiguration(classes = PersistenceSampleApplicationContext.class)
public class MatchDaoTest extends AbstractTransactionalTestNGSpringContextTests {
    @Autowired
    private MatchDao matchDao;

//    EntityManagerFactory emf = Persistence.createEntityManagerFactory("default");
//
//    @AfterEach
//    void afterEach(){
//        EntityManager em = null;
//        try {
//            em = emf.createEntityManager();
//            em.getTransaction().begin();
//            em.createQuery("delete from MatchEntity").executeUpdate();
//            em.getTransaction().commit();
//        } finally {
//            if (em != null) em.close();
//        }
//    }

    @Test
    void create() {
        MatchEntity match = new MatchEntity();
        matchDao.create(match);
        assertThat(match.getId()).isNotNull();
        Assertions.assertThat(matchDao.findAll()).hasSize(1);
    }

    @Test
    void findByIdExisting() {
        MatchEntity match = new MatchEntity();
        matchDao.create(match);
        var find = matchDao.findById(match.getId());
        assertThat(find).isEqualTo(match);
    }

    @Test
    void findByIdNonExisting() {
        Assertions.assertThat(matchDao.findAll()).isEmpty();

        var find = matchDao.findById(UUID.randomUUID());
        assertThat(find).isNull();
    }

    @Test
    void findAll() {
        MatchEntity matchA = new MatchEntity();
        MatchEntity matchB = new MatchEntity();
        matchDao.create(matchA);
        matchDao.create(matchB);
        Assertions.assertThat(matchDao.findAll()).hasSize(2);
    }

    @Test
    void update() {
        MatchEntity match = new MatchEntity();
        matchDao.create(match);
        TournamentEntity tournament = new TournamentEntity("Prievidza", CountryEnum.SLOVAKIA, 1, TournamentFormat.DEFAULT, 3);
        match.setTournament(tournament);
        TeamEntity teamA = new TeamEntity();
        teamA.setName("A");
        TeamEntity teamB = new TeamEntity();
        teamA.setName("B");
        match.setTeams(new HashSet<>(List.of(teamB,teamA)));

        matchDao.update(match);
        Assertions.assertThat(matchDao.findById(match.getId())).isEqualTo(match);
    }

    @Test
    void delete() {
        MatchEntity matchA = new MatchEntity();
        MatchEntity matchB = new MatchEntity();
        matchDao.create(matchA);
        matchDao.create(matchB);
        matchDao.remove(matchA);
        var matches = matchDao.findAll();
        Assertions.assertThat(matches).hasSize(1).contains(matchB);
    }

}
