package cz.muni.fi.pa165.dao;

import cz.muni.fi.pa165.PersistenceSampleApplicationContext;
import cz.muni.fi.pa165.dao.person.PersonDao;
import cz.muni.fi.pa165.entity.PersonEntity;
import cz.muni.fi.pa165.enums.CountryEnum;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit.jupiter.SpringExtension;
import org.springframework.test.context.testng.AbstractTransactionalTestNGSpringContextTests;
import org.springframework.transaction.annotation.Transactional;
import org.testng.annotations.Test;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import java.time.LocalDate;
import java.util.UUID;

import static org.assertj.core.api.Assertions.assertThat;

/**
 * @author Adrian Piacek
 */
@ContextConfiguration(classes = PersistenceSampleApplicationContext.class)
public class PersonDaoTest extends AbstractTransactionalTestNGSpringContextTests {
    @Autowired
    private PersonDao personDao;

//    EntityManagerFactory managerFactory = Persistence.createEntityManagerFactory("default");
//
//    @AfterEach
//    void afterEach() {
//        EntityManager manager = null;
//        try {
//            manager = managerFactory.createEntityManager();
//            manager.getTransaction().begin();
//            manager.createQuery("delete from PersonEntity ").executeUpdate();
//            manager.getTransaction().commit();
//        } finally {
//            if (manager != null) {
//                manager.close();
//            }
//        }
//    }

    @Test
    void create() {
        PersonEntity person = new PersonEntity("Johan");
        personDao.create(person);
        assertThat(person.getId()).isNotNull();
        assertThat(personDao.findAll()).hasSize(1);
    }

    @Test
    void findByIdExisting() {
        PersonEntity person = new PersonEntity("Johan");
        personDao.create(person);

        var find = personDao.findById(person.getId());
        assertThat(find).isEqualTo(person);
    }

    @Test
    void findByIdNonExisting() {
        assertThat(personDao.findAll()).isEmpty();

        var find = personDao.findById(UUID.randomUUID());
        assertThat(find).isNull();
    }

    @Test
    void findAll() {
        PersonEntity personA = new PersonEntity("PersonA");
        PersonEntity personB = new PersonEntity("PersonB");

        personDao.create(personA);
        personDao.create(personB);
        assertThat(personDao.findAll()).hasSize(2);
    }

    @Test
    void update() {
        PersonEntity personA = new PersonEntity("PersonA");
        personDao.create(personA);
        personA.setCountry(CountryEnum.AZERBAIJAN);
        personA.setDateOfBirth(LocalDate.of(2001,1,5));

        personDao.update(personA);
        assertThat(personDao.findById(personA.getId())).isEqualTo(personA);
    }

    @Test
    void delete() {
        PersonEntity personA = new PersonEntity("PersonA");
        PersonEntity personB = new PersonEntity("PersonB");
        personDao.create(personA);
        personDao.create(personB);
        personDao.remove(personA);
        var persons = personDao.findAll();
        assertThat(persons).hasSize(1).contains(personB);
    }
}
